//
//  ModifyWeightsViewController.swift
//  Measured Blood Lost
//
//  Created by Josh Birnholz on 4/13/16.
//  Copyright © 2016 Josh Birnholz. All rights reserved.
//

import UIKit

class AllWeightsViewController : UITableViewController, UIDocumentInteractionControllerDelegate {
	
	let userDefaults = NSUserDefaults.standardUserDefaults()
	
	var interactionController: UIDocumentInteractionController?
	
	var changed = false
	
	var allWeights: [Weight] = []
	
	override func viewDidLoad() {
		
		allWeights = unpackWeightsFromUserDefaults().sort { $0.name < $1.name }
		
		self.navigationItem.leftBarButtonItem = editButtonItem()
		
	}
	
	override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
		return allWeights.count
	}
	
	override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
		let cell = tableView.dequeueReusableCellWithIdentifier("AllWeightsCell", forIndexPath: indexPath)
		
		let weight = allWeights[indexPath.row]
		
		cell.textLabel!.text = weight.name
		cell.detailTextLabel!.text = "\(weight.weightInGrams) gm"
		
		return cell
	}
	
	func unpackWeightsFromUserDefaults() -> [Weight] {
		var error: String
		
		if let archivedWeights = userDefaults.valueForKey("archivedWeights") {
			print("Unpacking weights from user defaults.")
			
			if let data = archivedWeights as? NSData {
				if let allWeights = NSKeyedUnarchiver.unarchiveObjectWithData(data) as? [Weight] {
					return allWeights
				} else {
					error = "Unable to read data as weights."
				}
			} else {
				error = "Unable to open saved weights."
			}
			
		}
		error = "Saved weights did not exist."
		
		userDefaults.setObject(defaultWeights().archived, forKey: "archivedWeights")
		userDefaults.synchronize()
		
		print("Archived default weights to user defaults.")
		
		let alertController = UIAlertController(title: "Error", message: "An error occurred reading the saved weights: \(error)\n The weights have been set to the defaults and saved.", preferredStyle: .Alert)
		
		let cancelAction = UIAlertAction(title: "OK", style: UIAlertActionStyle.Cancel) { (action) in
			// Do nothing
		}
		alertController.addAction(cancelAction)
		
		if let popover = alertController.popoverPresentationController {
			popover.barButtonItem = doneButton
			popover.sourceView = self.view
		}
		
		self.presentViewController(alertController, animated: true) {
			
		}
		
		return allWeights
		
	}
	
	func packWeightsToUserDefaults() {
		let archivedObject = NSKeyedArchiver.archivedDataWithRootObject(allWeights as NSArray)
		userDefaults.setObject(archivedObject, forKey: "archivedWeights")
		userDefaults.synchronize()
		
		print("Archived default weights to user defaults.")
	}
	
	override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
		return true
	}
	
	override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
		if editingStyle == .Delete {
			// Remove data
			allWeights.removeAtIndex(indexPath.row)
			tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: UITableViewRowAnimation.Automatic)
		}
		changed = true
	}
	
	
	@IBOutlet var doneButton: UIBarButtonItem!
	
	@IBAction func saveButtonPressed(sender: AnyObject) {
		if changed {
			let alertController = UIAlertController(title: "Do you want to save your changes?", message: nil, preferredStyle: .ActionSheet)
			
			let cancelAction = UIAlertAction(title: "Cancel", style: UIAlertActionStyle.Cancel) { (action) in
				// Do nothing
			}
			alertController.addAction(cancelAction)
			
			let saveAction = UIAlertAction(title: "Save", style: UIAlertActionStyle.Default) { (actinon) in
				self.packWeightsToUserDefaults()
				self.performSegueWithIdentifier("unwinedFromListSegue", sender: self)
			}
			alertController.addAction(saveAction)
			
			let doNotSaveAction = UIAlertAction(title: "Don't Save", style: .Destructive) { (action) in
				self.performSegueWithIdentifier("unwinedFromListSegue", sender: self)
			}
			alertController.addAction(doNotSaveAction)
			
			if let popover = alertController.popoverPresentationController {
				popover.barButtonItem = doneButton
				popover.sourceView = self.view
			}
			
			self.presentViewController(alertController, animated: true) {
			}
		} else {
			self.performSegueWithIdentifier("unwinedFromListSegue", sender: self)
		}
	}
	
	@IBAction func resetButtonPressed(sender: AnyObject) {
		let alertController = UIAlertController(title: "Sync Weights", message: "Do you want to download weights from the server?", preferredStyle: .ActionSheet)
		
		let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel) { (action) in
			// Do nothing
		}
		alertController.addAction(cancelAction)
		
		let downloadAction = UIAlertAction(title: "Download", style: .Default) { (action) in
			if let weights = downloadWeightsFromServer() {
				self.allWeights = weights.unarchived
				
				self.allWeights.sortInPlace({ $0.name < $1.name })
				
				NSUserDefaults().setObject(weights.archived, forKey: "archivedWeights")
				NSUserDefaults().synchronize()
				
				self.tableView.reloadData()
				self.setEditing(false, animated: false)
				self.changed = false
			}
		}
		alertController.addAction(downloadAction)
		
		let resetAction = UIAlertAction(title: "Reset to Defaults", style: .Destructive) { (action) in
			let weights = defaultWeights()
			self.allWeights = weights.unarchived
			
			self.allWeights.sortInPlace({ $0.name < $1.name })
			
			NSUserDefaults().setObject(weights.archived, forKey: "archivedWeights")
			NSUserDefaults().synchronize()
			
			self.tableView.reloadData()
			self.setEditing(false, animated: false)
			self.changed = false
		}
		alertController.addAction(resetAction)
		
		if let popover = alertController.popoverPresentationController {
			popover.barButtonItem = resetButton
			popover.sourceView = self.view
		}
		
		self.presentViewController(alertController, animated: true) {
		}
	}
	
	override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
		performSegueWithIdentifier("SegueToWeightDetailModify", sender: self)
	}
	
	override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
		if segue.identifier == "SegueToWeightDetailModify" {
			let destination = segue.destinationViewController as? AddNewWeightViewController
			if let row = tableView.indexPathForSelectedRow?.row {
				
				destination!.allWeightsIndex = row
				
				destination!.name = allWeights[row].name
				destination!.weightInGrams = allWeights[row].weightInGrams
				
				if allWeights[row].category == nil {
					destination!.categoryOperatingRoom = true
					destination!.categoryVaginalDelivery = true
				} else if allWeights[row].category == .OperatingRoom {
					destination!.categoryOperatingRoom = true
				} else {
					destination!.categoryVaginalDelivery = true
				}
			}
			
			
		}
	}
	
	
	
	@IBOutlet var resetButton: UIBarButtonItem!
	
	override func setEditing(editing: Bool, animated: Bool) {
		super.setEditing(editing, animated: animated)
		
		if editing {
			navigationItem.setRightBarButtonItem(resetButton, animated: true)
		} else {
			navigationItem.setRightBarButtonItem(doneButton, animated: true)
		}
		
	}
	
	@IBAction func unwindToAllWeightsList(sender: UIStoryboardSegue) {
		
		if let source = sender.sourceViewController as? AddNewWeightViewController {
			
			
			if let weightToAdd = source.weight {
				
				var index = source.allWeightsIndex
				
				if index == nil {
					index = allWeights.count
				} else {
					allWeights.removeAtIndex(index!)
				}
				
				allWeights.insert(weightToAdd, atIndex: index!)
				
				allWeights.sortInPlace({ $0.name < $1.name })
				tableView.reloadData()
				changed = true
			}
		}
	}
	
	@IBAction func shareButtonPressed(sender: AnyObject) {
		
		let writeOperation = writeWeightsToFile()
		
		if writeOperation.success {
			print("Wrote file to \(writeOperation.filePath)")
			shareDocument(writeOperation.filePath)
		} else {
			print("Failed to write weights to file")
		}
		
	}
	
	func shareDocument(filePath: String) {
		
		let fileManager = NSFileManager()
		
		if fileManager.fileExistsAtPath(filePath) {
			
			let fileURL = NSURL.fileURLWithPath(filePath)
			
			interactionController = UIDocumentInteractionController(URL: fileURL)
			
			// (build your document's URL from its path in the Documents directory)
			
			interactionController!.delegate = self
			
			// First, attempt to show the "Open with... (app)" menu. Will fail and
			// do nothing if no app is present that can open the specified document
			// type.
			
			if interactionController!.presentOpenInMenuFromRect(self.view.frame, inView: self.view, animated: true) == true {
				
				// Fall back to "options" view:
				
				interactionController!.presentOptionsMenuFromRect(self.view.frame, inView: self.view, animated: true)
				print ("Showed options dialog")
				
			} else {
				print ("Showed open in other app dialog")
			}
			
		} else {
			print ("No file at specificed path")
		}
		
	}
	
	func writeWeightsToFile() -> (success: Bool, filePath: String) {
		let archivedObject = NSKeyedArchiver.archivedDataWithRootObject(allWeights as NSArray)
		
		let date = NSDate()
		let calendar = NSCalendar.currentCalendar()
		let components = calendar.components([.Hour, .Minute, .Second, .Day, .Month, .Year], fromDate: date)
		let year = components.year
		let month = components.month
		let day = components.day
		let hours = components.hour
		let minutes = components.minute
		let seconds = components.second
		
		NSDate().timeIntervalSince1970
		
		let fileName = "\(month)-\(day)-\(year) \(hours)-\(minutes)-\(seconds).weights"
		
		let directory = NSHomeDirectory().stringByAppendingPathComponent("Documents")
		
		let filePath = directory.stringByAppendingPathComponent(fileName)
		
		return (archivedObject.writeToFile(filePath, atomically: true), filePath)
	}
	
}

