//
//  SelectWeightsViewController.swift
//  Measured Blood Lost
//
//  Created by Josh Birnholz on 4/12/16.
//  Copyright © 2016 Josh Birnholz. All rights reserved.
//

import UIKit



class SelectWeightsViewController : UITableViewController, WeightCellDelegate {
	
	let userDefaults = NSUserDefaults.standardUserDefaults()
	
	var allWeights: [Weight] = []
	
	var availableWeights = [Weight]()
	
	var weightsToAdd = [Weight]()
	
	@IBOutlet weak var doneButton: UIBarButtonItem!
	@IBOutlet weak var cancelButton: UIBarButtonItem!
	
	@IBOutlet weak var categoryControl: UISegmentedControl!
	
	var keyboardIsUp = false
	
	
	override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
		setEditing(false, animated: true)
		if cancelButton === sender {
			weightsToAdd = [Weight]()
		}
	}
	
	
	override func viewDidLoad() {
		if let indexToSelect = userDefaults.valueForKey("categoryControlPreviousSelectedIndex") as? Int {
			categoryControl.selectedSegmentIndex = indexToSelect
		}
		
		allWeights = unpackWeightsFromUserDefaults()
		
		availableWeights = allWeights
		
		setListItems()
	
	}
	
	func setListItems() {
		availableWeights.removeAll()
		
		var category: Category
		
		if categoryControl.selectedSegmentIndex == 0 {
			category = .OperatingRoom
		} else {
			category = .VaginalDelivery
		}
		
		for weight in allWeights {
			if weight.category == nil || weight.category! == category {
				availableWeights.append(weight)
			}
		}
		
		availableWeights.sortInPlace { $0.name < $1.name }
		weightsToAdd = availableWeights
		tableView.reloadData()
	}
	
	func unpackWeightsFromUserDefaults() -> [Weight] {
		var error: String
		
		if let archivedWeights = userDefaults.valueForKey("archivedWeights") {
			print("Unpacking weights from user defaults.")
			
			if let data = archivedWeights as? NSData {
				if let allWeights = NSKeyedUnarchiver.unarchiveObjectWithData(data) as? [Weight] {
					for weight in allWeights {
						
						let name = weight.name
						let quantity = weight.quantity
						let weightInGrams = weight.weightInGrams
						let category = weight.category
						
						var categoryString: String
						
						if category == nil {
							categoryString = "not in a category."
						} else if category == .OperatingRoom {
							categoryString = "in category OperatingRoom"
						} else {
							categoryString = "in category VaginalDelivery"
						}
						
						print("Unpacking weight: \(quantity) \(name.pluralize(quantity)), weighing \(weightInGrams) gm each, \(categoryString)")
					}
					
					return allWeights
				} else {
					error = "Unable to read data as weights."
				}
			} else {
				error = "Unable to open saved weights."
			}
			
		}
		error = "Saved weights did not exist."
		
		
		let weights = downloadWeightsWithFallback()
		userDefaults.setObject(weights.archived, forKey: "archivedWeights")
		userDefaults.synchronize()
		
		print("Archived default weights to user defaults.")
		
		let alertController = UIAlertController(title: "Error", message: "An error occurred reading the saved weights: \(error)\n The weights have been reset to the defaults and saved.", preferredStyle: .Alert)
		
		let cancelAction = UIAlertAction(title: "OK", style: UIAlertActionStyle.Cancel) { (action) in
			// Do nothing
		}
		alertController.addAction(cancelAction)
		
		self.presentViewController(alertController, animated: true) {
			
		}
		
		return (weights.unarchived)
		
	}
	
	override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
		return 1
	}
	
	@IBAction func categoryControlValueChanged(sender: AnyObject) {
		setListItems()
		userDefaults.setValue(categoryControl.selectedSegmentIndex, forKey: "categoryControlPreviousSelectedIndex")
		userDefaults.synchronize()
	}
	
	override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
		return availableWeights.count
	}
	
	override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
		let cell = tableView.dequeueReusableCellWithIdentifier("WeightPickCell", forIndexPath: indexPath) as! WeightCell
		
		let name = availableWeights[indexPath.row].name
		let weightInGrams = availableWeights[indexPath.row].weightInGrams
		let quantity = availableWeights[indexPath.row].quantity
		
		if let nameLabel = cell.viewWithTag(100) as? UILabel {
			nameLabel.text = "\(name.pluralize(quantity))"
			nameLabel.textColor = quantity == 0 ? UIColor.lightGrayColor() : UIColor.darkTextColor()
		}
		if let weightLabel = cell.viewWithTag(101) as? UILabel {
			weightLabel.text = "\(weightInGrams) gm"
		}
		
		if let quantityTextField = cell.viewWithTag(102) as? UITextField {
			quantityTextField.text = quantity == 0 ? "" : String(quantity)
		}
		cell.delegate = self
		
		return cell
	}
	
	override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
		
		if view.endEditing(true)  {
			
			weightsToAdd[indexPath.row].quantity += 1
			tableView.reloadData()
		}
	}
	
	override func tableView(tableView: UITableView, shouldHighlightRowAtIndexPath indexPath: NSIndexPath) -> Bool {
		
		if view.endEditing(true) == false {
			return false
		}
		
		return true
		
	}
	
	func quantityChanged(newQuantity: Int, actionSender: AnyObject) {
		
		weightsToAdd[rowForSender(actionSender)].quantity = newQuantity
		tableView.reloadData()
		
	}
	
	func rowForSender(sender: AnyObject) -> Int {
		let point = sender.convertPoint(CGPointZero, toView: tableView)
		let indexPath = self.tableView.indexPathForRowAtPoint(point)!
		
		return indexPath.row
	}
	
}